# -*- encoding:utf- -*-

#file:		pyhttpd.py
#author:	zrking  @ wandoer.com
#date:		2016/11/19 22:30
#version:	0.1
#remark:	简单实现 http 服务器，用于学习 HTTP 协议。更多内容请访问 http://wandoer.com/coding/networks_protocol_http.htm
#

import os
from socket import *
from time import ctime

def GetResponseCommonHeader():
	"""获取通用的 Response 头 (已带CRLF).
	"""
	buf = 'Date:' + ctime() + '\r\n'
	buf = buf + 'Server:myHttpd\r\n'
	buf = buf + 'Content-Type:text/html\r\n'
	return buf

def GetResponseStatusLine(status):
	"""获取 Response 状态行(已带CRLF).
	"""
	httpVer = 'HTTP/1.1'
	return httpVer + ' ' + str(status) + ' ' + STATUS_CODE[status] + '\r\n'

def SendMsg(skt,buf):
	skt.send(buf)

def SendResponse(skt,status,body):
	"""发送 Response.
	
	Args:
		skt: socket
		status: 状态码
		body: Response 消息体
	"""
	buf = GetResponseStatusLine(status)
	buf = buf + GetResponseCommonHeader()
	buf = buf + '\r\n'
	buf = buf + body
	SendMsg(skt,buf)


def DoRenderPage(templateName,params = None,status = 200):
	"""由模板生成页面.
	
	Args:
		templateName: 请求的模板名称
		params: 请求参数列表({'Key=Valye','Key=Value'])
	"""
	global STATICPATH
	staticPath = STATICPATH
	status = 200
	if templateName == "/" or len(templateName) == 0:
		templateName = 'index.html'

	pagePath = staticPath+templateName

	print 'DorenderPage:',pagePath,params
	if not os.path.exists(pagePath):
		pagePath = staticPath+'404.html'
		status = 404
	
	return RenderPage(pagePath,params,status)


def RenderPage(templateName,params = None, status = 200):
	print 'RenderPage:', templateName,params
	template = open(templateName,'r')
	try:
		buf = template.read()
		if params != None:
			for p in params:
				pm = p.split('=')
				print pm
				if len(pm) != 2:
					raise Exception,'params error'
				pName = '${%s}' % (pm[0])
				buf = buf.replace(pName,pm[1])
	except Exception,e:
		print 'Exception : ',e
		buf = 'message=',str(e)
		global STATICPATH
		pagePath = STATICPATH + '500.html'
		return 500,''
	#	return RenderPage(pagePath,buf)
	finally:
		template.close()

	return status,buf

		
def DoGet(params):
	""" 响应 GET 请求.
	"""
	print 'DoGet:',params
	if params == None or len(params) == 0 or params == '/':
		return DoRenderPage('index.html')	
	
	lst = params.split('?')
	if len(lst) == 1:
		return DoRenderPage(lst[0])
	elif len(lst) >= 2:
		lstparam = lst[1].split('&')
		return DoRenderPage(lst[0],lstparam)		
	else:
		msg = "message=Please Check Your Input!"
		return DoRenderPage('400.html',msg,404)
		


def GetHttpRequest(data):
	""" 解析 HTTP 请求.

	return:
		list 0-方法  1-URI(及参数)
	"""
	buf = ''
	idx = 0
	c = data[idx]
	while c != '\n':
		buf = buf + c
		idx = idx + 1
		c = data[idx]
	if len(buf) < 3:
		return None
	lst = buf.split(' ')
	print lst
	if len(lst) < 3:
		return None
	return lst[0],lst[1]
	

#======================== config =====================

HOST = '127.0.0.1'  	#服务器地址
PORT = 9891				#服务端端口
BUFSZ = 10240			#支持最大 Request 长度
		
STATICPATH = './template/'	#静态资源路径

STATUS_CODE = {200:'OK',404:'Not Found',500:'InternalServerError'} #定义状态码及状态描述

#======================= main =======================

ADDR = (HOST,PORT)
skt = socket(AF_INET,SOCK_STREAM)		#建立socket
skt.setsockopt(SOL_SOCKET,SO_REUSEADDR,1)	#设置地址复用
skt.bind(ADDR)	#绑定地址

skt.listen(5)

while True:
	tcpClientSkt,addr = skt.accept()
	print 'accept from [%s,%s]' % (str(addr[0]),str(addr[1]))
	while True:
		try:
			data = tcpClientSkt.recv(BUFSZ)
		except Exception,e:
			print 'Exception',e
			tcpClientSkt.close()
			break
		if not data:
			break
		msg = '[%s] :\r\n%s' % (ctime(),data)
		print msg
		request = GetHttpRequest(data)
		if request == None:
			print 'Http request error'
		buf = ''
		method = request[0]
		if method != 'GET':
			param = ['message=method [%s] not implament yet!' % (method)]
			buf = DoRenderPage('500.html',param,500)
		else:
			buf = DoGet(request[1]) 
			
		SendResponse(tcpClientSkt,status=buf[0],body=buf[1])	
		tcpClientSkt.close()
skt.close()
